/* Author: kurt.stephens@parlano.com 2000/10/06 */
/*   parlano eventloop library -- pl_el/tevent.h
 *   Copyright (C) 2001 Parlano, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __pl_tevent_h
#define __pl_tevent_h

#ifndef pl_tevent_time

#include "timeval.h"

#define pl_tevent_time struct timeval

#define pl_tevent_time_now() pl_timeval_now()

#define pl_tevent_time_to_time_t(X) pl_timeval_to_time_t(X)
#define pl_tevent_time_to_usec(X) ((X).tv_usec) /* pl_timeval_to_usec(X) */

#define pl_time_t_to_tevent_time(X) pl_time_t_to_timeval(X)

#define pl_timeval_to_tevent_time(X) (X)
#define pl_tevent_time_to_timeval(X) (X)

#define pl_tevent_time_cmp(X,Y) pl_timeval_cmp((X),(Y))
#define pl_tevent_time_add(X,Y) pl_timeval_add((X),(Y))
#define pl_tevent_time_sub(X,Y) pl_timeval_sub((X),(Y))

#endif

/************************************************************************
 ** time event priority queue.
 ** Author: kurt.stephens@parlano.com
 */
 
/* Event handler. */

struct pl_tevent_q;
typedef struct pl_tevent_q pl_tevent_q;

#define pl_tevent_proc_DECL(X) void X (void *data1, void *data2, void *handle)
typedef pl_tevent_proc_DECL((*pl_tevent_proc));

typedef struct pl_tevent {
  /*@ A timed event for the queue. */
  
  struct pl_tevent *next;  
  /*@ Queue doubly-linked list header. */
  
  struct pl_tevent *prev;  
  /*@ Queue doubly-linked list header. */
  
  pl_tevent_time when;
  /*@ When to call %proc%(%data%). */

  pl_tevent_proc proc;
  /*@ The event action. */

  void *data2;
  /*@ Callback data2 argument. */

  pl_tevent_q *queue;
  /*@ The event queue it was added to. */

  void (*destroy)(void *entry);
  /*@ Destroys the event entry. */
} pl_tevent;


struct pl_tevent_q {
  /*@ A timed-event queue. */

  pl_tevent queue;
  /*@ The head of the queue. */

  int (*set_alarm)(pl_tevent_q *q, pl_tevent_time at_time, void (*func)());
  /*@ Set the alarm. */

  void *data1;
  /*@ Callback data1 argument. */

  void *opaque[2];
  /*@ Client data. */
};

#ifndef PL_TEVENT_IMPORT
#define PL_TEVENT_IMPORT extern
/*@ How to import pl_tevent decls. */
#endif

#ifndef PL_TEVENT_EXPORT
#define PL_TEVENT_EXPORT
/*@ How to export pl_tevent decls. */
#endif

PL_TEVENT_IMPORT int pl_tevent_q_init(pl_tevent_q *q);
PL_TEVENT_IMPORT int pl_tevent_q_destroy(pl_tevent_q *q);

PL_TEVENT_IMPORT int pl_tevent_q_add_event(pl_tevent_q *q, pl_tevent *e);
PL_TEVENT_IMPORT void *pl_tevent_q_add(pl_tevent_q *q, pl_tevent_time when, pl_tevent_proc proc, void *data);
PL_TEVENT_IMPORT int pl_tevent_q_remove(pl_tevent_q *q, void *handle);

PL_TEVENT_IMPORT int pl_tevent_q_check(pl_tevent_q *q, pl_tevent_time now);

#endif
