/* Author: kurt.stephens@parlano.com 2000/10/09 */
/*   parlano eventloop library -- pl_rr/buf.h
 *   Copyright (C) 2001 Parlano, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _pl_rr_buf_h
#define _pl_rr_buf_h

/*************************************************************************/

#include <stddef.h> /* size_t */

/*************************************************************************/

typedef enum pl_rr_buf_state {
  /*@ The state of a %pl_rr_buf%. */

  pl_rr_buf_START = 0,
  /*@ Request buffer is read for initialization. */
  
  pl_rr_buf_COMPLETE,
  /*@ Buffer is complete. */

  pl_rr_buf_IO_PENDING,
  /*@ Buffer is being sent/recv. */

  pl_rr_buf_IO_COMPLETE
  /*@ Buffer has been sent/recv. */

} pl_rr_buf_state;


struct pl_rr_buf;
typedef struct pl_rr_buf pl_rr_buf;

struct pl_rr_buf {
  /*@ Stores incoming and outgoing data. */

  char *buf;
  /*@ Beginning of allocated buffer. */

  size_t bufsize;
  /*@ Size of allocated buffer.  Actual allocation size of buf is %bufsize% + 1 for null terminator. */

  char *end;
  /*@ End of allocated buffer: %buf% + %bufsize%. */

  char *readp;
  /*@ Read position. */

  char *writep;
  /*@ Write position. */
  
  pl_rr_buf_state state;
  /*@ Buffer state. */

  pl_rr_id part;
  /*@ The request part. */
};

/*************************************************************************/

int pl_rr_buf_init(pl_rr_buf *rrb, pl_rr_id part, const char *buf, size_t len);
int pl_rr_buf_destroy(pl_rr_buf *rrb);

/*************************************************************************/
/* State. */

pl_rr_buf_state pl_rr_buf_get_state(pl_rr_buf *rrb);
int pl_rr_buf_close(pl_rr_buf *rrb);

/*************************************************************************/
/* Write. */

char * pl_rr_buf_write_ptr(pl_rr_buf *rrb);
size_t pl_rr_buf_write_len(pl_rr_buf *rrb);
int    pl_rr_buf_write_count(pl_rr_buf *rrb, size_t count);
int pl_rr_buf_write(pl_rr_buf *rrb, const char *buf, size_t len);

/*************************************************************************/
/* Read. */

char * pl_rr_buf_read_ptr(pl_rr_buf *rrb);
size_t pl_rr_buf_read_len(pl_rr_buf *rrb);
int    pl_rr_buf_read_count(pl_rr_buf *rrb, size_t count);

/*************************************************************************/
/* Internals. */

int pl_rr_buf_resize(pl_rr_buf *rrb, size_t bufsize);
int _pl_rr_buf_reset(pl_rr_buf *rrb);

/*************************************************************************/

#endif
