/* Author: kurt.stephens@parlano.com 2000/10/09 */
/*   parlano eventloop library -- pl_rr/endpoint.h
 *   Copyright (C) 2001 Parlano, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _pl_rr_endpoint_h
#define _pl_rr_endpoint_h

#include "hdr.h"

enum pl_rr_endpoint_state { 
  /*@ State of I/O on an endpoint. */

  pl_rr_ep_START = 0,
  /*@ Start state. */
  
  pl_rr_ep_HEADER,
  /*@ Currently reading or writing a message header. */
  
  pl_rr_ep_BODY
  /*@ Currently reading or writing a message body. */
};


struct pl_rr_endpoint {
  /*@ A source or destination of request response objects. */

  struct pl_rr_q *queue;
  /*@ Queue to handle incoming requests. */

  /*************************************************/
  /* Request handler. */

  pl_rr_handler reqh;
  /*@ Routine to call when an incoming request is available. */

  void *reqh_data;
  /*@ data for %reqh% callback. */

  /*************************************************/
  /* Methods. */

  int (*send)(pl_rr_endpoint *ep, const void *buf, size_t len, void *data);
  /*@ Send any OUTGOING data. */
  
  void *send_data;
  /*@ data for %send% method. */

  int (*recv)(pl_rr_endpoint *ep, void *buf, size_t len, void *data);
  /*@ Receive any INCOMING data. */

  void *recv_data;
  /*@ data for %receive% method. */

  int (*eos)(pl_rr_endpoint *ep);
  /*@ Called when send or recv return 0; end-of-stream condition. */

  int (*ioerror)(pl_rr_endpoint *ep);
  /*@ Called when send or recv returned -1; I/O error condition. */

  int (*sendable)(pl_rr_endpoint *ep, pl_rr *rr);
  /*@ If %rr% is non-null, %rr% has sendable (request or response) data. */

  /*************************************************/
  /* Current I/O state. */

  /* enum pl_rr_endpoint_state */
  int state[2];
  /*@ State of incoming/outgoing stream. */

  pl_rr_header hdr[2];
  /*@ Current header for incoming/outgoing rr. */

  pl_rr *rr[2];
  /*@ Currently active outgoing/incoming rr. */

  pl_rr_buf *rrb[2];
  /*@ Currently active req/res buffer. */

  /*************************************************/
  /* User-level. */

  int debug;
  /*@ Debug support. */

  void *opaque[2];
  /*@ Opaque data. */
};


int pl_rr_endpoint_init(pl_rr_endpoint *ep, pl_rr_q *queue, pl_rr_handler reqh, void *reqh_data);
int pl_rr_endpoint_destroy(pl_rr_endpoint *ep);

int pl_rr_endpoint_reschedule(pl_rr_endpoint *ep);
int pl_rr_endpoint_clear_pending(pl_rr_endpoint *ep);
int pl_rr_endpoint_recv(pl_rr_endpoint *ep);
int pl_rr_endpoint_send(pl_rr_endpoint *ep);

/*************************************************************************/
/* Internals. */

int _pl_rr_endpoint_sendable(pl_rr_endpoint *ep, pl_rr *rr);

#endif
