/* Author: kurt.stephens@parlano.com 2000/09/26 */
/*   parlano eventloop library -- pl_rr/fdobj_reqres.h
 *   Copyright (C) 2001 Parlano, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _pl_FDOBJ_REQRES_H
#define _pl_FDOBJ_REQRES_H


#include "reqres.h"
#include "pl_el/pl_el.h"

struct pl_rr_fdo;
typedef struct pl_rr_fdo pl_rr_fdo;

#define pl_rr_fdo_event_DECL(X) int X (pl_rr_fdo *rrfd)
typedef pl_rr_fdo_event_DECL((*pl_rr_fdo_event_handler));

struct pl_rr_fdo {
  /*@ A bridge between pl_rr_q, pl_rr_endpoint and pl_fd. */

  const char *name;
  /*@ Name for this endpoint. */

  pl_rr_fdo *srv;
  /*@ 
    %srv% points to bound server if this is an accepted client.
    0 if this is a client-side connection or server. 
  */

  pl_rr_endpoint _ep;
  /*@ Space for %ep%. */

  pl_rr_endpoint *ep;
  /*@ The req/res endpoint attached to %fdo%. */

  pl_fd _fdo;
  /*@ Space for %fdo%. */

  pl_fd *fdo;
  /*@ The fdobj we are attached to. */

  pl_el *el;
  /*@ The event loop. */

  pl_rr_q *queue;
  /*@ The request/response queue. */


  /***************************************************/
  /* Event handlers. */

  pl_rr_fdo_event_handler event_ioerror;
  /*@
    Called when any endpoint has an I/O error during
    transmission of requests or responses.
    After returning, 
    if the %pl_rr_fdo% is an accepted client, it is destroyed,
    if the %pl_rr_fdo% is a connecting client, it is rescheduled for reconnection if the event handler returns zero, otherwise it is closed.
  */

  pl_rr_fdo_event_handler event_eos;
  /*@
    Called when any endpoint read returns EOS.
    After returning,
    if the %pl_rr_fdo% is an accepted client, it is destroyed,
    if the %pl_rr_fdo% is a connecting client, it is rescheduled for reconnection if the event handler returns zero, otherwise it is closed.
    This is not nesssarly an "error" event.
    Accepted clients on the server will get EOS when 
    the client has no more requests.
    Connecting clients will get EOS when the server has disconnected them.
  */

  pl_rr_fdo_event_handler event_destroy;
  /*@
    Called once when %pl_rr_fdo_destroy% is called.
  */

  /***************************************************/
  /* Server/accepted client events. */

  pl_rr_fdo_event_handler event_accepting;
  /*@ 
    Called with the newly created pl_fdo_rr accepted client object
    when a client is accepting by a server.
    If returns non-zero, the connecting client is dropped. 
    If you are validating client IP addresess, return non-zero to drop the connecting client.
  */

  pl_rr_fdo_event_handler event_accepted;
  /*@ 
    Called with a pl_rr_fdo accepted client object when the client has been 
    acceptined by a server.  
    If returns non-zero, the connecting client is dropped. 
  */
 
  pl_rr_fdo_event_handler event_accept_error;
  /*@ 
    Called with a pl_rr_fdo accepted client object
    when the completion of a client connect fails.
    After returning the accepted client is destroyed.
   */

  /***************************************************/
  /* Connecting client events. */

  pl_rr_fdo_event_handler event_connecting;
  /*@ Called when a client is connecting to a server. */

  pl_rr_fdo_event_handler event_connect;
  /*@ Called when a client is connected to a server. */

  pl_rr_fdo_event_handler event_connect_error;
  /*@ Called when a client connect error happens, before retring. */

  pl_rr_fdo_event_handler event_connect_retry;
  /*@ Called when a client connection retry is retried. */

  pl_rr_fdo_event_handler event_connect_retry_error;
  /*@
    Called when a client connection retry is terminated. 
    Note: the event-loop is no longer stopped on sto
  */

  void (*dealloc)(void *);
  /*@ Deallocation. */

  int debug;
  /*@ Debugging. */

  void *opaque[2];
  /*@ Client data. */
};


/***************************************************************/


int pl_rr_fdo_init_srv
(
 pl_rr_fdo *fdrr, 
 pl_rr_q *queue,
 pl_rr_handler reqh, 
 void *reqh_data, 
 const char *srv_conn, 
 pl_el *el,
 pl_fd *cli_proto
 );

int pl_rr_fdo_init_cli
(
 pl_rr_fdo *fdrr, 
 pl_rr_q *queue, 
 pl_rr_handler reqh, 
 void *reqh_data,
 const char *cli_conn, 
 pl_el *el,
 int reconnect_delay 
 );

int pl_rr_fdo_destroy(pl_rr_fdo *fdrr);

pl_rr_fdo *pl_fd_rr_fdo(pl_fd *fdo);
pl_rr_fdo *pl_rr_endpoint_rr_fdo(pl_rr_endpoint *ep);

#endif
