/* Author: kurt.stephens@parlano.com 2000/09/28 */
/*   parlano eventloop library -- pl_rr/hdr.h
 *   Copyright (C) 2001 Parlano, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _pl_rr_header_h
#define _pl_rr_header_h

#include <stddef.h> /* size_t */

struct pl_rr_header;
typedef struct pl_rr_header pl_rr_header;

struct pl_rr_header {
  char proto;
  short op;
  unsigned long id;
  unsigned long part;
  size_t len;

  /* 
     +-+-+----+----+----+-----------
     |P|O|ID  |PART|LEN |LEN bytes ...
     +-+-+----+----+----+-----------
     
     P     1 BYTE
        0
	The message framing protocol, currently zero

     O     1 BYTE
        { REQUEST | RESPONSE };

     ID    4 bytes, big-endian
        The request or response ID.

     PART  4 bytes, big-endian
        Which part of the request or response, 0 means last part or single part message.

     LEN   4 bytes, big-endian
        How long the part is.

     total: 14 bytes
  */

  unsigned char buf[16];
    /*@ Encoded header buffer. */
  unsigned char *end;
    /*@ End of buffer pointer. */
  unsigned char *rp;
    /*@ Read pointer. */
  unsigned char *wp;
    /*@ Write pointer. */

};

int pl_rr_header_init(pl_rr_header *h);

int pl_rr_header_encode(pl_rr_header *h);
int pl_rr_header_decode(pl_rr_header *h);

#endif
