/* Author: kurt.stephens@parlano.com 2000/10/09 */
/*   parlano eventloop library -- pl_rr/queue.h
 *   Copyright (C) 2001 Parlano, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _pl_rr_q_h
#define _pl_rr_q_h

#include "pl_el/dlinklist.h" /* pl_dll_BEGIN(), pl_dll_END() */

/*************************************************************************/
/* Event handler declarator. */

#define pl_rr_q_event_handler_DECL(X) int X (pl_rr_q *rrq)
typedef pl_rr_q_event_handler_DECL((*pl_rr_q_event_handler));

/*************************************************************************/

struct pl_rr_q {
  /*@ A queue of request/response objects. */

  pl_rr head;
  /*@ Head for the queue. */

  int count;
  /*@ Number of reqrest/response objects in queue. */

  pl_rr_q_event_handler event_empty;
  /*@ Called once when queue is empty. */

  int debug;
  /*@ Debug support. */

  void *opaque[2];
  /*@ Opaque data. */
};


/*************************************************************************/
/* Initialization. */

int pl_rr_q_init(pl_rr_q *rrq);
int pl_rr_q_destroy(pl_rr_q *rrq);


/*************************************************************************/
/* Events. */

int pl_rr_q_set_event_empty(pl_rr_q *rrq, pl_rr_q_event_handler eh);


/*************************************************************************/
/* Query. */

pl_rr *pl_rr_q_get_by_id(pl_rr_q *rrq, pl_rr_id id);


/*************************************************************************/
/* Request Creation. */

pl_rr *pl_rr_q_req_open(pl_rr_q *rrq, pl_rr_endpoint *dst, pl_rr_handler resh, void *data);


/*************************************************************************/
/* Iteration. */

/* C++-style iterators. */

pl_rr *pl_rr_q_begin(pl_rr_q *rrq);
#define pl_rrr_q_begin(Q) pl_dll_BEGIN(&((Q)->head))

pl_rr *pl_rr_q_end(pl_rr_q *rrq);
#define pl_rr_q_end(Q) pl_dll_END(&((Q)->head))


/* Remember rr->next because removal of rr will side-effect it. */
#define pl_rr_q_LOOP(Q,rr) \
do { \
  pl_rr *rr##_next; \
  for ( (rr) = pl_rr_q_begin((Q)); (rr) != pl_rr_q_end((Q)); (rr) = rr##_next ) \
  { \
     rr##_next = (rr)->next; \
    {

#define pl_rr_q_LOOP_END() \
    } \
  } \
} while(0)

/*************************************************************************/
/* Debugging tools. */

int pl_rr_q_dump(pl_rr_q *rrq, FILE *fp);
int pl_rr_dump(pl_rr *rr, FILE *fp);
int pl_rr_buf_dump(pl_rr_buf *rrb, const char *prefix, FILE *fp);

/*************************************************************************/
/* Internals. */

int _pl_rr_q_remove(pl_rr_q *rrq, pl_rr *rr);

pl_rr *_pl_rr_q_next_sendable(pl_rr_q *rrq, pl_rr_endpoint *ep);
pl_rr *_pl_rr_q_begin_read(pl_rr_q *rrq, int op, pl_rr_id id, pl_rr_id part, size_t len, pl_rr_endpoint *ep);
int _pl_rr_q_complete(pl_rr_q *rrq, pl_rr *rr);

int _pl_rr_q_add_to_end(pl_rr_q *rrq, pl_rr *rr);
int _pl_rr_q_add_to_begin(pl_rr_q *rrq, pl_rr *rr);

/*************************************************************************/

#endif
