/* Author: kurt.stephens@parlano.com 2000/10/05 */
/*   parlano eventloop library -- pl_el/callback.c
 *   Copyright (C) 2001 Parlano, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "callback.h"

#include "dlinklist.h"
#include <stdlib.h>
#include <assert.h>

int pl_cb_init(pl_cb *cbl)
     /*@ Initializes callback list %cbl%. */
{
  pl_dll_INIT(cbl);

  return 0;
}

#define cbl_LOOP(cbl) { \
  pl_cb *cb, *cbn; \
  for ( cb = pl_dll_BEGIN(cbl); cb != pl_dll_END(cbl); cb = cbn ) { \
    cbn = cb->next; \

#define cbl_LOOP_END() } }


int pl_cb_remove(void *handle)
     /*@ Removes callback %handle% from it's callback list. */
{
  pl_cb *cb = handle;

  if ( cb && cb->next ) {
    pl_dll_REMOVE(cb);
    cb->next = 0;
    free(cb);
  }

  return 0;
}


int pl_cb_destroy(pl_cb *cbl)
     /*@ Destroys all callbacks in callback list %cbl%. */
{
  cbl_LOOP(cbl);
  {
    pl_cb_remove(cb);
  }
  cbl_LOOP_END();

  return 0;
}


void *pl_cb_add(pl_cb *cbl, pl_cb_handler func, void *data2, int flags)
     /*@ Adds an new callback to %func% to %cbl%. 
       If %flags% contains %pl_cb_REPEAT%, %func% will be called for every %pl_cb_all%.
       Otherwise %func% will only be called once.
     */
{
  pl_cb *cb;

  assert(cbl);
  assert(func);

  cb = malloc(sizeof(*cb));
  assert(cb);

  pl_dll_INIT(cb);

  cb->func = func;
  cb->data2 = data2;
  cb->flags = flags;

  pl_dll_INSERT_BEFORE(cb, cbl);

  return cb;
}


int pl_cb_all(pl_cb *cbl, void *data1)
     /*@ Calls all callback functions in %cbl% with the %data1% argument. */
{
  int result = 0;

  cbl_LOOP(cbl);
  {
    /* Do callback. */
    result = cb->func(data1, cb->data2, (void*) cb);

    /* One-shot? */
    if ( ! (cb->flags & pl_cb_REPEAT) ) {
      pl_cb_remove(cb);
    }
  }
  cbl_LOOP_END();

  return result;
}


#undef cbl_LOOP
#undef cbl_LOOP_END
