/* Author: kurt.stephens@parlano.com 2000/10/09 */
/*   parlano eventloop library -- pl_el/timeval.c
 *   Copyright (C) 2001 Parlano, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "timeval.h"

#define USECS 1000000

struct timeval pl_timeval_now()
{
  struct timeval x;

  gettimeofday(&x, 0);

  return x;
}


struct timeval pl_double_to_timeval(double t)
     /*@
       Converts from double to struct timeval.
     */
{
  struct timeval x;

  x.tv_sec = (long) t;
  if ( t < 0 )
    t = - t;
  x.tv_usec = t - (double) (long) t;

  return x;
}


double pl_timeval_to_double(struct timeval x)
     /*@
       Converts from struct timeval to double.
     */
{
  return (double) x.tv_sec + (double) x.tv_usec / (double) USECS;
}


struct timeval pl_time_t_to_timeval(time_t t)
     /*@
       Converts from time_t to struct timeval.
     */
{
  struct timeval x;
  memset(&x, 0, sizeof(x));
  x.tv_sec = t;
  return x;
}


time_t pl_timeval_to_time_t(struct timeval t)
     /*@
       Converts from struct timeval to time_t.
      */
{
  return t.tv_sec;
}




int pl_timeval_cmp(struct timeval x, struct timeval y)
     /*@
       Compares %x% and %y%.
       Return -1, 0, or 1.
     */
{
  /* Carry. */
  while ( x.tv_usec < y.tv_usec && x.tv_sec ) {
    x.tv_usec += USECS;
    -- x.tv_sec;
  }

  if ( x.tv_sec < y.tv_sec )
    return -1;

  if ( x.tv_sec > y.tv_sec )
    return 1;

  if ( x.tv_usec < y.tv_usec )
    return -1;

  if ( x.tv_usec > y.tv_usec )
    return 1;

  return 0;
}


struct timeval pl_timeval_sub(struct timeval x, struct timeval y)
     /*@
       Subtract %y% from %x%.
      */
{
  if ( pl_timeval_cmp(x, y) > 0 ) {
    /* Carry. */
    while ( x.tv_usec < y.tv_usec && x.tv_sec ) {
      x.tv_usec += USECS;
      -- x.tv_sec;
    }

    /* Subtract. */
    x.tv_sec -= y.tv_sec;
    x.tv_usec -= y.tv_usec;
  } else {
    /* Clamp. */
    x.tv_sec = x.tv_usec = 0;
  }

  return x;
}



struct timeval pl_timeval_add(struct timeval x, struct timeval y)
     /*@
       Add %x% to %y%.
      */
{
  /* Add microsecs. */
  x.tv_usec += y.tv_usec;

  /* Do carry. */
  while ( x.tv_usec >= USECS && x.tv_sec ) {
    x.tv_usec -= USECS;
    ++ x.tv_sec;
  }

  /* Do secs. */
  x.tv_sec += y.tv_sec;

  return x;
}

#undef USECS
