/* Author: kurt.stephens@parlano.com 2000/09/28 */
/*   parlano eventloop library -- pl_rr/hdr.c
 *   Copyright (C) 2001 Parlano, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "hdr.h"
#include <assert.h>
#include <stdlib.h>


int pl_rr_header_encode(pl_rr_header *h)
{
  unsigned char *s;

  assert(h);

  h->rp = h->wp = h->buf;

  s = h->buf;
  
  *(s ++) = h->proto;

  *(s ++) = h->op;

  *(s ++) = (((unsigned long) h->id) >>  0) & 0xff;
  *(s ++) = (((unsigned long) h->id) >>  8) & 0xff;
  *(s ++) = (((unsigned long) h->id) >> 16) & 0xff;
  *(s ++) = (((unsigned long) h->id) >> 24) & 0xff;

  *(s ++) = (((unsigned long) h->part) >>  0) & 0xff;
  *(s ++) = (((unsigned long) h->part) >>  8) & 0xff;
  *(s ++) = (((unsigned long) h->part) >> 16) & 0xff;
  *(s ++) = (((unsigned long) h->part) >> 24) & 0xff;

  *(s ++) = (((unsigned long) h->len) >>  0) & 0xff;
  *(s ++) = (((unsigned long) h->len) >>  8) & 0xff;
  *(s ++) = (((unsigned long) h->len) >> 16) & 0xff;
  *(s ++) = (((unsigned long) h->len) >> 24) & 0xff;

  h->end = s;
  assert(h->end - h->buf <= sizeof(h->buf));

  return 0;
}


int pl_rr_header_decode(pl_rr_header *h)
{ 
  unsigned char *s;

  h->rp = h->wp = h->buf;

  s = h->buf;

  h->rp = h->buf;

  h->proto = *(s ++);
  
  h->op = *(s ++);
      
  h->id = 
    (((unsigned long) s[0]) <<  0) |
    (((unsigned long) s[1]) <<  8) |
    (((unsigned long) s[2]) << 16) |
    (((unsigned long) s[3]) << 24) ;
  s += 4;
  
  h->part = 
    (((unsigned long) s[0]) <<  0) |
    (((unsigned long) s[1]) <<  8) |
    (((unsigned long) s[2]) << 16) |
    (((unsigned long) s[3]) << 24) ;
  s += 4;
      
  h->len = 
    (((unsigned long) s[0]) <<  0) |
    (((unsigned long) s[1]) <<  8) |
    (((unsigned long) s[2]) << 16) |
    (((unsigned long) s[3]) << 24) ;
  s += 4;

  h->end = s;

  assert(h->end - h->buf <= sizeof(h->buf));

  h->wp = s;

  return 0;
}


int pl_rr_header_init(pl_rr_header *h)
{
  assert(h);

  memset(h, 0, sizeof(*h));
  pl_rr_header_encode(h); /* compute h->end. */

  return 0;
}

